package at.jku.ssw.psw2.ue01.demos.list;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

/**
 * 
 * Main class for the person list demo application. 
 * Sets up the ui and starts the application. 
 * 
 * @author hp
 *
 */
public class PersonListDemoApp {

	private static JFrame frame;
	private static PersonList list;
	private static JButton addBtn;
	private static JButton removeBtn;
	private static JButton updBtn;
	private static JButton sortBtn;
	private static JTextField nameField;
	private static ButtonGroup sexGroup;
	private static JCheckBox maleCheck;
	private static JCheckBox femaleCheck;
	private static PersonModel model;

	/**
	 * Main method starting the application. 
	 * @param args not used 
	 */
	public static void main(String[] args) {

		SwingUtilities.invokeLater(() -> {
			frame = createFrame();
			frame.pack();
			frame.setLocation(200, 200);
			frame.setVisible(true);
		});
	}

	/**
	 * Creates the main frame of the application. 
	 * Sets up the whole ui. 
	 * 
	 * @return the main frame of the application. 
	 */
	private static JFrame createFrame() {
		JFrame frame = new JFrame("Hallo Persons");
		frame.getContentPane().setLayout(new BorderLayout());
		frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

		// create person model and list component 
		model = new PersonModel();
		list = new PersonList(model);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		frame.getContentPane().add(new JScrollPane(list), BorderLayout.CENTER);

		// setting up the input controls and buttons 
		JPanel northPnl = new JPanel();
		frame.getContentPane().add(northPnl, BorderLayout.NORTH);
		nameField = new JTextField(12);
		sexGroup = new ButtonGroup();
		maleCheck = new JCheckBox("Male");
		maleCheck.setSelected(true);
		femaleCheck = new JCheckBox("Female");
		sexGroup.add(maleCheck);
		sexGroup.add(femaleCheck);
		addBtn = new JButton("Add");
		removeBtn = new JButton("Remove");
		updBtn = new JButton("Update");
		sortBtn = new JButton("Sort");
		northPnl.add(nameField);
		northPnl.add(maleCheck);
		northPnl.add(femaleCheck);
		northPnl.add(addBtn);
		northPnl.add(removeBtn);
		northPnl.add(updBtn);
		northPnl.add(sortBtn);

		// handling the add button by adding a person to the list
		addBtn.addActionListener(a -> {
			String name = nameField.getText();
			if (name.length() == 0) {
				return;
			}
			Sex sex = (maleCheck.isSelected() ? Sex.Male : Sex.Female);
			model.addPerson(new Person(name, sex));
		});

		// handling the remove button by removing the selected a person from the list
		removeBtn.addActionListener(a -> {
			int selected = list.getSelectedIndex();
			if (selected < 0) {
				return;
			}
			model.removePerson(selected);
		});

		
		// handling the update button by changing the data of the selected a person
		updBtn.addActionListener(a -> {
			int selected = list.getSelectedIndex();
			if (selected < 0) {
				return;
			}
			String name = nameField.getText();
			if (name.length() == 0) {
				return;
			}
			Sex sex = (maleCheck.isSelected() ? Sex.Male : Sex.Female);
			model.updatePerson(selected, name, sex);
		});

		// handling the sort button by sorting the list of persons 
		sortBtn.addActionListener(a -> {
			model.sort();
		});

		// reacting to changes of the selection in the list
		list.addListSelectionListener(e -> {
			int idx = list.getSelectedIndex();
			if (idx < 0) {
				return;
			}
			Person p = model.get(idx);
			if (p == null) {
				return;
			}
			nameField.setText(p.getName());
			if (p.getSex() == Sex.Male) {
				maleCheck.setSelected(true);
			} else {
				femaleCheck.setSelected(true);
			}
		});

		// setting the renderer for the list elements 
		list.setCellRenderer(new PersonCellCompRenderer());

		return frame;
	}

	/**
	 * Renderer class for rendering the values in the person list. 
	 * Rendering is done by printing the name of the person in different colors 
	 * dependent on the sex of the person, i.e., blue for male and red for female. 
	 *  
	 * @author hp
	 */
	@SuppressWarnings("unused")
	private static class PersonCellRenderer implements ListCellRenderer<Person> {

		private static final Color MALE_COLOR = Color.BLUE;
		private static final Color FEMALE_COLOR = Color.RED;

		private final JLabel label;

		/**
		 * Constructor setting up the label used as rendering component. 
		 */
		PersonCellRenderer() {
			label = new JLabel();
			label.setOpaque(true);
		}

		/* (non-Javadoc)
		 * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
		 */
		@Override
		public Component getListCellRendererComponent(JList<? extends Person> list, Person value, int index,
				boolean isSelected, boolean cellHasFocus) {
			label.setText(value.getName());
			label.setForeground(value.getSex() == Sex.Male ? MALE_COLOR : FEMALE_COLOR);
			if (isSelected) {
				label.setBackground(list.getSelectionBackground());
			} else {
				label.setBackground(list.getBackground());
			}
			return label;
		}

	}

	/**
	 * Renderer class for rendering the values in the person list. 
	 * Rendering is done by painting a red or blue dot dependent on the sex 
	 * and displaying the name of the person. Additionally the position of the 
	 * value in the list is displayed. 
	 *  
	 * @author hp
	 */
	private static class PersonCellCompRenderer implements ListCellRenderer<Person> {

		private static final Color MALE_COLOR = Color.BLUE;
		private static final Color FEMALE_COLOR = Color.RED;

		private final JPanel panel;
		private final JLabel nameLbl;
		private final JLabel idxLbl;
		private final DotComponent dotComp;

		/**
		 * Constructor setting up the panel used as rendering component. 
		 */
		PersonCellCompRenderer() {
			panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
			nameLbl = new JLabel();
			idxLbl = new JLabel();
			dotComp = new DotComponent();
			panel.setOpaque(true);
			panel.add(idxLbl);
			panel.add(dotComp);
			panel.add(nameLbl);
		}

		/* (non-Javadoc)
		 * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
		 */
		@Override
		public Component getListCellRendererComponent(JList<? extends Person> list, Person value, int index,
				boolean isSelected, boolean cellHasFocus) {
			idxLbl.setText(String.valueOf(index + 1));
			nameLbl.setText(value.getName());
			dotComp.setColor(value.getSex() == Sex.Male ? MALE_COLOR : FEMALE_COLOR);
			if (isSelected) {
				panel.setBackground(list.getSelectionBackground());
			} else {
				panel.setBackground(list.getBackground());
			}
			return panel;
		}

	}

	/**
	 * A component painting the red or blue dot to show the sex of a person. 
	 * 
	 * @author hp
	 */
	@SuppressWarnings("serial")
	private static class DotComponent extends JComponent {

		Color color;

		{
			setPreferredSize(new Dimension(20, 20));
		}

		/* (non-Javadoc)
		 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
		 */
		@Override
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			g.setColor(color);
			g.fillOval(0, 0, getWidth(), getHeight());
		}

		/**
		 * Sets the color used for rendering the dot. 
		 * @param color the color used for painting the dot. 
		 */
		void setColor(Color color) {
			this.color = color;
		}

	}

}
