package at.jku.ssw.psw2.ue01.demos.list;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.AbstractListModel;
import javax.swing.JList;
import javax.swing.ListModel;

/**
 * Model for person objects represented in a {@link JList}. Allows changing a
 * list of persons. Implements {@link ListModel} for person objects.
 * 
 * @author hp
 *
 */
@SuppressWarnings("serial")
public class PersonModel extends AbstractListModel<Person> {

	private final List<Person> persons;

	/**
	 * Default constructor initializing the list to an empty list.
	 */
	public PersonModel() {
		super();
		this.persons = new ArrayList<>();

	}

	/**
	 * Adds a person to the list of persons, if not already contained.
	 * 
	 * @param p
	 *            the person to add
	 */
	public void addPerson(Person p) {
		if (persons.contains(p)) {
			return;
		}
		persons.add(p);
		fireIntervalAdded(this, persons.size() - 1, persons.size() - 1);
	}

	/**
	 * Removes a person at a certain position.
	 * 
	 * @param idx
	 *            the position of the person to remove in the list.
	 */
	public void removePerson(int idx) {
		persons.remove(idx);
		fireIntervalRemoved(this, idx, idx);
	}

	/**
	 * Updates the data of the person at the given index.
	 * 
	 * @param idx
	 *            the position of the person in the list
	 * @param name
	 *            the new name of the person
	 * @param sex
	 *            the possibly changed sex of the person
	 */
	public void updatePerson(int idx, String name, Sex sex) {
		if (idx < 0 || idx >= persons.size()) {
			return;
		}
		persons.remove(idx);
		persons.add(idx, new Person(name, sex));
		fireContentsChanged(this, idx, idx);
	}

	/**
	 * Sorts the person list based on the name.
	 */
	public void sort() {
		Collections.sort(persons, (p1, p2) -> {
			return p1.getName().compareTo(p2.getName());
		});
		fireContentsChanged(this, 0, size() - 1);
	}

	/**
	 * Returns the current size of the person list.
	 * 
	 * @return the size of the person list.
	 */
	public int size() {
		return persons.size();
	}

	/**
	 * Gets the person at position <code>idx</code> in the list.
	 * 
	 * @param idx
	 *            the position of the person to get
	 * @return returns the person object at position <code>idx</code> in the list.
	 */
	public Person get(int idx) {
		return persons.get(idx);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.ListModel#getSize()
	 */
	@Override
	public int getSize() {
		return size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.ListModel#getElementAt(int)
	 */
	@Override
	public Person getElementAt(int index) {
		return get(index);
	}

}
